package se.hellsoft.syncloud;

import android.app.Notification;
import android.app.Service;
import android.content.Intent;
import android.media.MediaPlayer;
import android.net.Uri;
import android.net.wifi.WifiManager;
import android.os.*;
import android.util.Log;
import com.android.volley.Response;
import com.android.volley.toolbox.NetworkImageView;
import org.java_websocket.WebSocket;
import org.json.JSONArray;
import org.json.JSONException;
import org.json.JSONObject;
import se.hellsoft.syncloud.network.CommComponentTCP;
import se.hellsoft.syncloud.soundcloud.SoundCloudAPI;
import se.hellsoft.syncloud.soundcloud.SoundCloudPlayer;

import java.net.InetAddress;
import java.net.UnknownHostException;

/**
 * @author Erik Hellman
 */
public class BackgroundService extends Service implements CommComponentTCP.CommListener, SoundCloudPlayer.PlayerListener {
    public static final String TAG = "BackgroundService";
    private static final String HOST_ADDRESS = "172.16.13.89";
    private static final int SONY_ADJUSTMENT = -50;
    private LocalBinder mLocalBinder = new LocalBinder();
    private CommComponentTCP mCommComponent = null;
    private SoundCloudAPI mSoundCloudAPI;
    private SynCloudData.Playlist mPlaylist;
    private boolean mHost;
    private SynCloudData.SynMessage.Builder mSynMessageBuilder;
    private int mCurrentTrack = -1;
    private SoundCloudPlayer mSoundCloudPlayer;
    private PlaylistListener mPlaylistListener;
    private boolean mIfSony;
    private PowerManager.WakeLock mWakeLock;

    public IBinder onBind(Intent intent) {
        return mLocalBinder;
    }

    @Override
    public void onCreate() {
        super.onCreate();
        mSoundCloudAPI = new SoundCloudAPI();
        mSoundCloudAPI.init(this);
        Log.d(TAG, "onCreate!");
        mIfSony = Build.MANUFACTURER.equals("Sony");
    }

    @Override
    public void onDestroy() {
        mSoundCloudAPI.destroy();
        super.onDestroy();
        Log.d(TAG, "onDestroy!");
    }

    public void startComponent(boolean isHost) {
        mHost = isHost;

        PowerManager powerManager = (PowerManager) getSystemService(POWER_SERVICE);
        PowerManager.WakeLock wakeLock = powerManager.newWakeLock(PowerManager.FULL_WAKE_LOCK, TAG);
        wakeLock.acquire();
        mWakeLock = wakeLock;

        try {
            WifiManager wifiManager = (WifiManager) getSystemService(WIFI_SERVICE);
            int ipAddress = wifiManager.getConnectionInfo().getIpAddress();
            byte[] addressBytes = intToByteArray(ipAddress);
            InetAddress inetAddress = InetAddress.getByAddress(addressBytes);
            if (mCommComponent == null) {
                mCommComponent = new CommComponentTCP();
                mCommComponent.addListener(this);
                if (isHost) {
                    mSynMessageBuilder = SynCloudData.SynMessage.newBuilder();
                    mPlaylist = SynCloudData.Playlist.newBuilder().
                            setHostAddress(inetAddress.getHostAddress()).setVersion(1).build();
                    mCommComponent.startServer(inetAddress);
                } else {
                    mCommComponent.connectClient(InetAddress.getByName(HOST_ADDRESS));
                }
            }

            startForeground(1, buildNotification());
        } catch (UnknownHostException e) {
            Log.e(TAG, "Error starting comm component...", e);
        }

        mSoundCloudPlayer = new SoundCloudPlayer(this);
        mSoundCloudPlayer.setPlayerListener(this);
    }

    public static byte[] intToByteArray(int value) {
        return new byte[]{
                (byte) value,
                (byte) (value >>> 8),
                (byte) (value >>> 16),
                (byte) (value >>> 24)};
    }

    private Notification buildNotification() {
        Notification.Builder builder = new Notification.Builder(this);
        builder.setSmallIcon(R.drawable.ic_launcher);
        builder.setContentTitle(getString(R.string.app_name));
        return builder.build();
    }

    public boolean isStarted() {
        return mCommComponent != null;
    }

    public boolean isHost() {
        return mHost;
    }

    public void shutdownComponent() {
        if(mWakeLock != null && mWakeLock.isHeld()) {
            mWakeLock.release();
        }

        if (mCommComponent != null) {
            mCommComponent.removeListener(this);
            if (mHost) {
                mCommComponent.stopServer();
            } else {
                mCommComponent.disconnectClient();
            }
        }
        mCommComponent = null;

        if (mSoundCloudPlayer != null) {
            mSoundCloudPlayer.setPlayerListener(null);
            mSoundCloudPlayer.stopPlayback();
        }
        mSoundCloudPlayer = null;

        stopForeground(true);
    }

    public void addTrack(JSONObject trackObject) {
        try {
            SynCloudData.Track.Builder trackBuilder = SynCloudData.Track.newBuilder();
            long id = trackObject.getLong("id");
            trackBuilder.setId(id);
            String title = trackObject.getString("title");
            trackBuilder.setTitle(title);
            trackBuilder.setUser("A User");
            long duration = trackObject.getLong("duration");
            trackBuilder.setDuration(duration);
            String streamUrl = trackObject.getString("stream_url");
            trackBuilder.setStreamUrl(streamUrl);
            SynCloudData.Track track = trackBuilder.build();

            SynCloudData.Playlist.Builder playlistBuilder = SynCloudData.Playlist.newBuilder();
            playlistBuilder.setHostAddress("a.b.c.d");
            playlistBuilder.setVersion(1);
            if (mPlaylist != null && mPlaylist.getTracksCount() > 0) {
                playlistBuilder.addAllTracks(mPlaylist.getTracksList());
            }
            playlistBuilder.addTracks(track);
            mPlaylist = playlistBuilder.build();
            mCommComponent.sendMessage(mSynMessageBuilder.setPlaylist(mPlaylist).
                    setType(SynCloudData.SynMessage.Type.PLAYLIST).build());
            if (mPlaylistListener != null) {
                mPlaylistListener.onPlaylistUpdated(mPlaylist);
            }
        } catch (JSONException e) {
            Log.e(TAG, "Error parsing JSON!", e);
        }
    }

    public void searchTracks(String query, Response.Listener<JSONArray> listener) {
        mSoundCloudAPI.searchTracks(query, listener);
    }

    public NetworkImageView getNetworkImageView(String url) {
        return mSoundCloudAPI.getNetworkImageView(url, this);
    }

    public void enqueueTrack(SynCloudData.Track track) {
        if (mSoundCloudPlayer != null) {
            String uri = track.getStreamUrl() + "?client_id=" + Constants.SOUNDCLOUD_CLIENT_ID;
//            String uri = "http://api.soundcloud.com/tracks/118153066/stream?client_id=010335534542aa219428a79ad33b2081";
            Log.d(TAG, "Play " + uri);
            long playTime = 5000;
            SynCloudData.SynMessage synMessage = mSynMessageBuilder.clear()
                    .setType(SynCloudData.SynMessage.Type.PLAY_TRACK)
                    .setTrack(track).setTime(playTime).build();
            mCommComponent.sendMessage(synMessage);
            long adjustment = mIfSony ? SONY_ADJUSTMENT : 0;
            mSoundCloudPlayer.enqueueTrack(Uri.parse(uri), SystemClock.elapsedRealtime() + playTime + 25  + adjustment);
        }
    }

    public void enqueueTrack(SynCloudData.Track track, long playTime) {
        if (mSoundCloudPlayer != null) {
            String uri = track.getStreamUrl() + "?client_id=" + Constants.SOUNDCLOUD_CLIENT_ID;
//            String uri = "http://api.soundcloud.com/tracks/118153066/stream?client_id=010335534542aa219428a79ad33b2081";
            Log.d(TAG, "Play " + uri + " at " + playTime);
            long adjustment = mIfSony ? SONY_ADJUSTMENT : 0;
            mSoundCloudPlayer.enqueueTrack(Uri.parse(uri), SystemClock.elapsedRealtime() + playTime + adjustment);
        }
    }

    @Override
    public void onMessageReceived(SynCloudData.SynMessage synMessage) {
        Log.d(TAG, "Received: " + synMessage.getType().name());
        switch (synMessage.getType().getNumber()) {
            case SynCloudData.SynMessage.Type.PLAY_TRACK_VALUE:
                enqueueTrack(synMessage.getTrack(), synMessage.getTime());
                break;
            case SynCloudData.SynMessage.Type.PLAYLIST_VALUE:
                mPlaylist = synMessage.getPlaylist();
                if(mPlaylistListener != null) {
                    mPlaylistListener.onPlaylistUpdated(mPlaylist);
                }
                break;
            case SynCloudData.SynMessage.Type.STOP_TRACK_VALUE:
                if(mSoundCloudPlayer != null) {
                    mSoundCloudPlayer.stopPlayback();
                }
        }
    }

    @Override
    public void onClientConnected(WebSocket webSocket) {
        Log.d(TAG, "Client connected!" + webSocket.getRemoteSocketAddress().getHostName());
        mSynMessageBuilder.clear().setPlaylist(mPlaylist).setType(SynCloudData.SynMessage.Type.PLAYLIST);
        webSocket.send(mSynMessageBuilder.build().toByteArray());
    }

    @Override
    public void onTimeSync(long diffFromHost) {
        //To change body of implemented methods use File | Settings | File Templates.
    }

    public void setPlaylistListener(PlaylistListener playlistListener) {
        mPlaylistListener = playlistListener;
    }

    public SynCloudData.Playlist getPlaylist() {
        return mPlaylist;
    }

    public void stopPlayback() {
        if(mSoundCloudPlayer != null) {
            mSoundCloudPlayer.stopPlayback();
        }

        if (mHost) {
            SynCloudData.SynMessage synMessage = mSynMessageBuilder.clear().setType(SynCloudData.SynMessage.Type.STOP_TRACK).build();
            mCommComponent.sendMessage(synMessage);
        }
    }

    @Override
    public void onMediaPlayerChanged(MediaPlayer mediaPlayer) {
        if (!mHost && mPlaylistListener != null) {
            mPlaylistListener.onMediaPlayerUpdated(mediaPlayer, mediaPlayer != null && mediaPlayer.isPlaying());
        }
    }

    public void sendUpdates() {
        if(!mHost && mPlaylistListener != null && mSoundCloudPlayer != null) {
            MediaPlayer mediaPlayer = mSoundCloudPlayer.getCurrentMediaPlayer();
            if (mediaPlayer != null) {
                mPlaylistListener.onMediaPlayerUpdated(mediaPlayer, mediaPlayer.isPlaying());
            }
        }

       if(mPlaylistListener != null) {
           mPlaylistListener.onPlaylistUpdated(mPlaylist);
       }
    }

    public class LocalBinder extends Binder {
        public BackgroundService getService() {
            return BackgroundService.this;
        }
    }

    public interface PlaylistListener {
        void onPlaylistUpdated(SynCloudData.Playlist playlist);
        void onMediaPlayerUpdated(MediaPlayer mediaPlayer, boolean playing);
    }
}
