package se.hellsoft.syncloud;

import android.app.Activity;
import android.app.Fragment;
import android.content.ComponentName;
import android.content.Intent;
import android.content.ServiceConnection;
import android.media.MediaPlayer;
import android.os.Bundle;
import android.os.IBinder;
import android.util.Log;
import android.view.Menu;
import android.view.MenuItem;
import android.view.View;
import android.view.ViewGroup;
import android.widget.*;
import com.android.volley.Response;
import org.json.JSONArray;
import org.json.JSONException;
import org.json.JSONObject;

public class MainActivity extends Activity implements ServiceConnection,
        Response.Listener<JSONArray>,BackgroundService.PlaylistListener {
    private static final String TAG = "MainActivity";
    private BackgroundService mService;
    private JSONArray mSearchResult;
    private SynCloudData.Playlist mPlaylist;
    private SearchFragment mSearchFragment;
    private SlaveFragment mSlaveFragment;
    private PlaylistFragment mPlaylistFragment;
    private SearchResultAdapter mSearchResultAdapter;
    private ListView mSearchResultList;
    private PlaylistAdapter mPlaylistAdapter;

    /**
     * Called when the activity is first created.
     */
    @Override
    public void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.main);

        mPlaylist = SynCloudData.Playlist.getDefaultInstance();

        mPlaylistFragment = new PlaylistFragment();
        mPlaylistAdapter = new PlaylistAdapter();
        mPlaylistFragment.setPlayListAdapter(mPlaylistAdapter);
        mSlaveFragment = new SlaveFragment();
        mSearchFragment = new SearchFragment();
        mSearchFragment.setPlaylistFragment(mPlaylistFragment);
        mSearchResultAdapter = new SearchResultAdapter();
        mSearchFragment.setSearchResultAdapter(mSearchResultAdapter);

        if (savedInstanceState == null) {
            getFragmentManager().beginTransaction()
                    .add(R.id.container, new StartupFragment())
                    .commit();
        }

        boolean result = bindService(new Intent(this, BackgroundService.class),
                this, BIND_AUTO_CREATE);
        Log.d(TAG, "Bind operation " + (result ? "successful!" : "failed!"));
    }

    @Override
    protected void onResume() {
        super.onResume();
    }

    @Override
    protected void onPause() {
        super.onPause();
    }

    @Override
    protected void onDestroy() {
        super.onDestroy();

        mService.setPlaylistListener(null);

        if (mService != null) {
            unbindService(this);
        }
    }

    @Override
    public boolean onCreateOptionsMenu(Menu menu) {

        // Inflate the menu; this adds items to the action bar if it is present.
        getMenuInflater().inflate(R.menu.menu, menu);

        return true;
    }

    public void doStartGroup(View view) {
        getFragmentManager().beginTransaction().
                replace(R.id.container, mPlaylistFragment).commit();
        mService.startComponent(true);
    }

    public void doJoinGroup(View view) {
        getFragmentManager().beginTransaction().
                replace(R.id.container, mSlaveFragment).commit();
        mService.startComponent(false);
    }

    public void doStopPlayback(MenuItem menuItem) {
        mService.stopPlayback();
    }

    public void doShowSearch(MenuItem menuItem) {
        Log.d(TAG, "Show search!");
        getFragmentManager().beginTransaction().
                replace(R.id.container, mSearchFragment).commit();
    }

    public void doExit(MenuItem menuItem) {
        if(mService != null) {
            mService.shutdownComponent();
        }
        finish();
    }

    public void doSearchTracks(View view) {
        Log.d(TAG, "Search tracks!");
        EditText editText = (EditText) mSearchFragment.getView().findViewById(R.id.search_query);
        mService.searchTracks(editText.getText().toString(), this);
    }

    @Override
    public void onServiceConnected(ComponentName name, IBinder service) {
        Log.d(TAG, "Service connected!");
        mService = ((BackgroundService.LocalBinder) service).getService();
        mPlaylist = mService.getPlaylist();
        mPlaylistAdapter.notifyDataSetChanged();
        mService.setPlaylistListener(this);
        mSearchFragment.setBackgroundService(mService);
        mPlaylistFragment.setBackgroundService(mService);
        mService.sendUpdates();
    }

    @Override
    public void onServiceDisconnected(ComponentName name) {
        Log.d(TAG, "Service disconnected!");
        mService = null;
    }

    @Override
    public void onResponse(JSONArray response) {
        mSearchResult = response;
        mSearchResultAdapter.notifyDataSetChanged();
    }

    @Override
    public void onPlaylistUpdated(SynCloudData.Playlist playlist) {
        mPlaylist = playlist;
        mPlaylistAdapter.notifyDataSetChanged();
    }

    @Override
    public void onMediaPlayerUpdated(MediaPlayer mediaPlayer, boolean isPlaying) {
        mSlaveFragment.setMediaPlayer(mediaPlayer,isPlaying);
    }


    class SearchResultAdapter extends BaseAdapter {

        @Override
        public int getCount() {
            if (mSearchResult != null) {
                return mSearchResult.length();
            } else {
                return 0;
            }
        }

        @Override
        public Object getItem(int position) {
            if (mSearchResult != null) {
                return mSearchResult.optJSONObject(position);
            } else {
                return null;
            }
        }

        @Override
        public long getItemId(int position) {
            JSONObject jsonObject = (JSONObject) getItem(position);
            return jsonObject.optInt("id");
        }

        @Override
        public View getView(int position, View convertView, ViewGroup parent) {
            if (convertView == null) {
                convertView = View.inflate(MainActivity.this, R.layout.track_item, null);
            }
            JSONObject jsonObject = (JSONObject) getItem(position);
            String title = jsonObject.optString("title");
            int duration = jsonObject.optInt("duration");
            ((TextView) convertView.findViewById(R.id.track_title_duration)).setText(title + " (" + duration / 1000 + "s)");
            String user = jsonObject.optString("user.username");
            ((TextView) convertView.findViewById(R.id.track_user)).setText(user);
//            String artworkUrl = jsonObject.optString("user.avatar_url");
            return convertView;
        }
    }

    class PlaylistAdapter extends BaseAdapter {

        @Override
        public int getCount() {
            if (mPlaylist != null) {
                return mPlaylist.getTracksCount();
            } else {
                return 0;
            }
        }

        @Override
        public Object getItem(int position) {
            return mPlaylist.getTracks(position);
        }

        @Override
        public long getItemId(int position) {
            return mPlaylist.getTracks(position).getId();
        }

        @Override
        public View getView(int position, View convertView, ViewGroup parent) {
            if (convertView == null) {
                convertView = View.inflate(MainActivity.this, R.layout.track_item, null);
            }
            SynCloudData.Track track = mPlaylist.getTracks(position);
            ((TextView) convertView.findViewById(R.id.track_title_duration)).setText(track.getTitle() + " (" + track.getDuration() / 1000 + "s)");
            ((TextView) convertView.findViewById(R.id.track_user)).setText(track.getUser());
//            String artworkUrl = jsonObject.optString("user.avatar_url");
            return convertView;
        }
    }

}
