package se.hellsoft.syncloud;

import android.content.Context;
import android.opengl.GLES20;
import android.opengl.GLSurfaceView;
import android.opengl.Matrix;
import android.os.SystemClock;
import android.util.Log;

import javax.microedition.khronos.egl.EGLConfig;
import javax.microedition.khronos.opengles.GL10;
import java.nio.ByteBuffer;
import java.nio.ByteOrder;
import java.nio.FloatBuffer;
import java.nio.ShortBuffer;
import java.util.Arrays;
import java.util.Random;

/**
 * User: Erik Westenius
 * Date: 01/11/13
 * Time: 19:14
 */
public class SlaveRenderer implements GLSurfaceView.Renderer {
    private static final String TAG = "SlaveRenderer";

    private Square mSquare;

    private Context mContext;

    private final float[] mMVPMatrix = new float[16];
    private final float[] mProjMatrix = new float[16];
    private final float[] mVMatrix = new float[16];
    private final float[] mRotationMatrix = new float[16];

    private String mRawFragmentShader = null;

    // Declare as volatile because we are updating it from another thread
    public volatile float mAngle;
    private Random mRandom = new Random();
    private int mWidth = 0;
    private int mHeight = 0;
    private int mSoundData = 0;
    private byte[] mFFTData;
    private float mScale = 0.1f;

    public void setScale(float scale) {
        mScale = scale;
    }
//    private AudioCapture mAudioCapture;

    public SlaveRenderer(String fragmentShader) {
        mRawFragmentShader = fragmentShader;
    }


    @Override
    public void onSurfaceCreated(GL10 unused, EGLConfig config) {

        // Set the background frame color
        GLES20.glClearColor(0.0f, 0.0f, 0.0f, 1.0f);

        mSquare = new Square(mRawFragmentShader);
//        mAudioCapture = new AudioCapture(AudioCapture.TYPE_PCM, 1024);
//        mAudioCapture.start();
    }

    @Override
    public void onDrawFrame(GL10 unused) {

        // Draw background color
        GLES20.glClear(GLES20.GL_COLOR_BUFFER_BIT);

        // Set the camera position (View matrix)
        Matrix.setLookAtM(mVMatrix, 0, 0, 0, -0.1f, 0f, 0f, 0f, 0f, 1.0f, 0.0f);
        // Calculate the projection and view transformation
        Matrix.multiplyMM(mMVPMatrix, 0, mProjMatrix, 0, mVMatrix, 0);

        // Draw square
        mSquare.draw(mMVPMatrix);

        // Create a rotation for the triangle
//        long time = SystemClock.uptimeMillis() % 4000L;
//        float angle = 0.090f * ((int) time);
//        Matrix.setRotateM(mRotationMatrix, 0, mAngle, 0, 0, -1.0f);

        // Combine the rotation matrix with the projection and camera view
//        Matrix.multiplyMM(mMVPMatrix, 0, mRotationMatrix, 0, mMVPMatrix, 0);

        // Draw triangle
//        mTriangle.draw(mMVPMatrix);
    }

    @Override
    public void onSurfaceChanged(GL10 unused, int width, int height) {
        // Adjust the viewport based on geometry changes,
        // such as screen rotation
        GLES20.glViewport(0, 0, width, height);

        mWidth = width;
        mHeight = height;
        float ratio = (float) width / height;

        // this projection matrix is applied to object coordinates
        // in the onDrawFrame() method
        Matrix.orthoM(mProjMatrix, 0, -ratio, ratio, -1, 1, 3, 7);
    }

    public static int loadShader(int type, String shaderCode) {

        // create a vertex shader type (GLES20.GL_VERTEX_SHADER)
        // or a fragment shader type (GLES20.GL_FRAGMENT_SHADER)
        int shader = GLES20.glCreateShader(type);

        // add the source code to the shader and compile it
        GLES20.glShaderSource(shader, shaderCode);
        GLES20.glCompileShader(shader);

        return shader;
    }

    public static void checkGlError(String glOperation) {
        int error;
        while ((error = GLES20.glGetError()) != GLES20.GL_NO_ERROR) {
            Log.e(TAG, glOperation + ": glError " + error);
            throw new RuntimeException(glOperation + ": glError " + error);
        }
    }

    public void setSoundData(int i) {
        mSoundData = i;
    }

    public void setFFTData(byte[] fft) {
        mFFTData = fft;
    }


    class Square {

        private final String vertexShaderCode =
                // This matrix member variable provides a hook to manipulate
                // the coordinates of the objects that use this vertex shader
                "uniform mat4 uMVPMatrix;" +

                        "attribute vec4 vPosition;" +
                        "void main() {" +
                        // the matrix must be included as a modifier of gl_Position
                        "  gl_Position = vPosition * uMVPMatrix;" +
                        "}";

//    private final String fragmentShaderCode =
//            "precision mediump float;" +
//                    "uniform vec4 vColor;" +
//                    "void main() {" +
//                    "  gl_FragColor = vColor;" +
//                    "}";

        private final FloatBuffer vertexBuffer;
        private final ShortBuffer drawListBuffer;
        private final int mProgram;
        private final String mFragmentShaderCode;
        private int mPositionHandle;
        private int mMVPMatrixHandle;
        private int mTimeHandle;
        private int mResolutionHandle;
        private int mRandomHandle;
        private long mStartTime = 0;
        private int mSoundDataHandle;
        private int mBassHandle;
        private int mScaleHandle;

        // number of coordinates per vertex in this array
        static final int COORDS_PER_VERTEX = 3;
        private float squareCoords[] = {-2f, 1f, 0.0f,   // top left
                -2f, -1f, 0.0f,   // bottom left
                2f, -1f, 0.0f,   // bottom right
                2f, 1f, 0.0f}; // top right

        private final short drawOrder[] = {0, 1, 2, 0, 2, 3}; // order to draw vertices

        private final int vertexStride = COORDS_PER_VERTEX * 4; // 4 bytes per vertex

        // Set color with red, green, blue and alpha (opacity) values
        float color[] = {0.2f, 0.709803922f, 0.898039216f, 1.0f};
        float resolution[] = {0, 0};
        private int [] mVizData = new int[1024];
        public Square(String fragmentShaderCode) {
            mFragmentShaderCode = fragmentShaderCode;
            // initialize vertex byte buffer for shape coordinates
            ByteBuffer bb = ByteBuffer.allocateDirect(
                    // (# of coordinate values * 4 bytes per float)
                    squareCoords.length * 4);
            bb.order(ByteOrder.nativeOrder());
            vertexBuffer = bb.asFloatBuffer();
            vertexBuffer.put(squareCoords);
            vertexBuffer.position(0);

            // initialize byte buffer for the draw list
            ByteBuffer dlb = ByteBuffer.allocateDirect(
                    // (# of coordinate values * 2 bytes per short)
                    drawOrder.length * 2);
            dlb.order(ByteOrder.nativeOrder());
            drawListBuffer = dlb.asShortBuffer();
            drawListBuffer.put(drawOrder);
            drawListBuffer.position(0);
            mStartTime = SystemClock.uptimeMillis();

            // prepare shaders and OpenGL program
            int vertexShader = SlaveRenderer.loadShader(GLES20.GL_VERTEX_SHADER,
                    vertexShaderCode);
            int fragmentShader = SlaveRenderer.loadShader(GLES20.GL_FRAGMENT_SHADER,
                    mFragmentShaderCode);

            mProgram = GLES20.glCreateProgram();             // create empty OpenGL Program
            GLES20.glAttachShader(mProgram, vertexShader);   // add the vertex shader to program
            GLES20.glAttachShader(mProgram, fragmentShader); // add the fragment shader to program
            GLES20.glLinkProgram(mProgram);                  // create OpenGL program executables

        }

        public void draw(float[] mvpMatrix) {
            // Add program to OpenGL environment
            resolution[0] = mWidth;
            resolution[1] = mHeight;
            float dbValue = 0;
            if(mFFTData != null && mFFTData.length > 0){
                byte rfk = mFFTData[10];
                byte ifk = mFFTData[10 + 1];
                long magnitude = (rfk * rfk + ifk * ifk);
                dbValue = (float) (10 * Math.log10(magnitude));
            }
            dbValue = Math.abs(dbValue) > 100 ? 0 : dbValue;
//            Log.d(TAG, "FFT DATA!!! " + dbValue);
//            if (mAudioCapture != null){
//                mVizData = mAudioCapture.getFormattedData(1, 1);
//            }else {
//                Arrays.fill(mVizData, (int) 0);
//            }
//            if(mVizData.length > 0)
//                Log.d(TAG, "HERPDERP: " + mVizData[0]);
            GLES20.glUseProgram(mProgram);
            float time = (SystemClock.uptimeMillis() - mStartTime);
            time /= 1000.0;
            time = time %1;

            // get handle to vertex shader's vPosition member
            mPositionHandle = GLES20.glGetAttribLocation(mProgram, "vPosition");
            

            // Enable a handle to the triangle vertices
            GLES20.glEnableVertexAttribArray(mPositionHandle);

            // Prepare the triangle coordinate data
            GLES20.glVertexAttribPointer(mPositionHandle, COORDS_PER_VERTEX,
                    GLES20.GL_FLOAT, false,
                    vertexStride, vertexBuffer);

            mTimeHandle = GLES20.glGetUniformLocation(mProgram, "iTime");
            SlaveRenderer.checkGlError("glGetUniformLocation");

            mResolutionHandle = GLES20.glGetUniformLocation(mProgram, "iResolution");
            SlaveRenderer.checkGlError("glGetUniformLocation");

            mRandomHandle = GLES20.glGetUniformLocation(mProgram, "iRandom");
            SlaveRenderer.checkGlError("glGetUniformLocation");

            mSoundDataHandle = GLES20.glGetUniformLocation(mProgram, "iSoundData");
            SlaveRenderer.checkGlError("glGetUniformLocation");

            mBassHandle = GLES20.glGetUniformLocation(mProgram, "iBass");
            SlaveRenderer.checkGlError("glGetUniformLocation");

            mScaleHandle = GLES20.glGetUniformLocation(mProgram, "iScale");
            SlaveRenderer.checkGlError("glGetUniformLocation");

            float random = mRandom.nextFloat();
            GLES20.glUniform1f(mRandomHandle, random);
            SlaveRenderer.checkGlError("glUniform1f");

            GLES20.glUniform2fv(mResolutionHandle, 1, resolution, 0);
            SlaveRenderer.checkGlError("glUniform2fv");

            GLES20.glUniform1f(mTimeHandle, time);
            SlaveRenderer.checkGlError("glUniform1f");

            GLES20.glUniform1f(mSoundDataHandle, mSoundData);
            SlaveRenderer.checkGlError("glUniform1f");

            GLES20.glUniform1f(mBassHandle, dbValue);
            SlaveRenderer.checkGlError("glUniform1f");

            GLES20.glUniform1f(mScaleHandle, mScale);
            SlaveRenderer.checkGlError("glUniform1f");

            // get handle to shape's transformation matrix
            mMVPMatrixHandle = GLES20.glGetUniformLocation(mProgram, "uMVPMatrix");
            SlaveRenderer.checkGlError("glGetUniformLocation");

            // Apply the projection and view transformation
            GLES20.glUniformMatrix4fv(mMVPMatrixHandle, 1, false, mvpMatrix, 0);
            SlaveRenderer.checkGlError("glUniformMatrix4fv");

            // Draw the square
            GLES20.glDrawElements(GLES20.GL_TRIANGLES, drawOrder.length,
                    GLES20.GL_UNSIGNED_SHORT, drawListBuffer);

            // Disable vertex array
            GLES20.glDisableVertexAttribArray(mPositionHandle);
        }
    }
}
