package se.hellsoft.syncloud.network;

import android.os.Handler;
import android.os.HandlerThread;
import android.os.Message;
import android.util.Log;
import com.google.protobuf.ByteString;
import se.hellsoft.syncloud.SynCloudData;

import java.io.IOException;
import java.net.*;
import java.util.Collections;
import java.util.HashSet;
import java.util.Set;

/**
 * @author Erik Hellman
 */
public class CommComponent implements Handler.Callback {
    private static final String TAG = "CommComponent";
    // Should be between 224.0.0.0 and 239.255.255.255
    public static final String MULTICAST_GROUP = "224.2.76.24";
    public static final int MULTICAST_PORT = 5500;
    private static final int MAX_MESSAGE_SIZE = 8192;
    private static final int SEND_MULTICAST_MSG = 10;
    private static final int BROADCAST_PLAYLIST_MSG = 20;

    private MulticastSocket mMulticastSocket;
    private DatagramSocket mDatagramSocket;
    private boolean mRunServer;
    private Handler mNetworkHandler;
    private DatagramPacket mSendPacket;
    private boolean mHost;
    private Set<CommListener> mCommListeners;
    private SynCloudData.Playlist mPlaylist;

    public CommComponent(boolean host) {
        if (host) {
            SynCloudData.Playlist.Builder playlistBuilder = SynCloudData.Playlist.newBuilder();
            playlistBuilder.setHostAddress("a.b.c.d");
            playlistBuilder.setVersion(1);
            mPlaylist = playlistBuilder.build();
        }
        mHost = host;
        byte[] data = new byte[MAX_MESSAGE_SIZE];
        mSendPacket = new DatagramPacket(data, MAX_MESSAGE_SIZE);
        mCommListeners = Collections.synchronizedSet(new HashSet<CommListener>());
    }

    public void startServer() {
        try {
            mRunServer = true;

/*
            NetworkInterface wifiInterface = null;
            Enumeration<NetworkInterface> networkInterfaces = NetworkInterface.getNetworkInterfaces();
            while (networkInterfaces.hasMoreElements()) {
                NetworkInterface networkInterface = networkInterfaces.nextElement();
                Log.d(TAG, "IF: " + networkInterface.getDisplayName());
                if("wlan0".equals(networkInterface.getDisplayName())) {
                    wifiInterface = networkInterface;
                    break;
                }
            }
*/

            HandlerThread handlerThread = new HandlerThread("NetworkSender");
            handlerThread.start();
            mNetworkHandler = new Handler(handlerThread.getLooper(), this);

            mMulticastSocket = new MulticastSocket(MULTICAST_PORT);
            mMulticastSocket.setLoopbackMode(true);
            mMulticastSocket.joinGroup(InetAddress.getByName(MULTICAST_GROUP));

            new Thread(new MulticastThread()).start();

            if(mHost) {
                mNetworkHandler.sendEmptyMessageDelayed(BROADCAST_PLAYLIST_MSG, 5000);
            }
        } catch (IOException e) {
            Log.e(TAG, "Error creating multicast socket!", e);
        }
    }

    public void stopServer() {
        mRunServer = false;
        try {
            mNetworkHandler.getLooper().quit();
        } catch (Exception e) {
            // Ignore
        }
        mNetworkHandler = null;

        try {
            mMulticastSocket.close();
        } catch (Exception e) {
            // Ignore
        }
        mMulticastSocket = null;
    }

    public void sendMulticastMessage(SynCloudData.SynMessage synMessage) {
        mNetworkHandler.obtainMessage(SEND_MULTICAST_MSG, synMessage).sendToTarget();
    }

    @Override
    public boolean handleMessage(Message msg) {
        switch (msg.what) {
            case SEND_MULTICAST_MSG:
                try {
                    SynCloudData.SynMessage synMessage = (SynCloudData.SynMessage) msg.obj;
                    byte[] data = synMessage.toByteArray();
                    DatagramPacket datagramPacket = new DatagramPacket(data, data.length,
                            InetAddress.getByName(MULTICAST_GROUP), MULTICAST_PORT);
                    mMulticastSocket.send(datagramPacket);
                } catch (IOException e) {
                    Log.e(TAG, "Error sending multicast packet!", e);
                }
                break;
            case BROADCAST_PLAYLIST_MSG:
                try {
                    SynCloudData.SynMessage synMessage = SynCloudData.SynMessage.newBuilder().
                            setPlaylist(mPlaylist).setType(SynCloudData.SynMessage.Type.PLAYLIST).build();
                    byte[] data = synMessage.toByteArray();
                    DatagramPacket datagramPacket = new DatagramPacket(data, data.length,
                            InetAddress.getByName(MULTICAST_GROUP), MULTICAST_PORT);
                    mMulticastSocket.send(datagramPacket);
                } catch (IOException e) {
                    Log.e(TAG, "Error sending multicast packet!", e);
                }
                if(mHost && mRunServer) {
                    mNetworkHandler.sendEmptyMessageDelayed(BROADCAST_PLAYLIST_MSG, 5000);
                }
                break;


        }
        return true;
    }

    public boolean isHost() {
        return mHost;
    }

    public void setHost(boolean host) {
        mHost = host;
    }

    public SynCloudData.Playlist getPlaylist() {
        return mPlaylist;
    }

    public void setPlaylist(SynCloudData.Playlist playlist) {
        mPlaylist = playlist;
    }

    private class MulticastThread implements Runnable {

        @Override
        public void run() {
            try {
                byte[] data = new byte[MAX_MESSAGE_SIZE];
                DatagramPacket datagramPacket = new DatagramPacket(data, data.length,
                        InetAddress.getByName(MULTICAST_GROUP), MULTICAST_PORT);
                while(mRunServer) {
                    mMulticastSocket.receive(datagramPacket);
                    Log.d(TAG, "Received multicast package!!!");
                    SynCloudData.SynMessage synMessage = SynCloudData.SynMessage.
                            parseFrom(ByteString.copyFrom(datagramPacket.getData(), 0, datagramPacket.getLength()));
                    notifyListeners(synMessage);
                }
            } catch (IOException e) {
                Log.e(TAG, "Error running mutlicast server!", e);
            }
        }
    }

    public void addListener(CommListener commListener) {
        mCommListeners.add(commListener);
    }

    public void removeListener(CommListener commListener) {
        mCommListeners.remove(commListener);
    }

    private void notifyListeners(SynCloudData.SynMessage synMessage) {
        for (CommListener commListener : mCommListeners) {
            commListener.onMessageReceived(synMessage);
        }
    }

    public interface CommListener {
        void onMessageReceived(SynCloudData.SynMessage synMessage);
    }
}
