package se.hellsoft.syncloud.network;

import android.os.Handler;
import android.os.HandlerThread;
import android.os.Message;
import android.os.SystemClock;
import android.util.Log;
import com.google.protobuf.ByteString;
import com.google.protobuf.InvalidProtocolBufferException;
import org.java_websocket.WebSocket;
import org.java_websocket.client.WebSocketClient;
import org.java_websocket.handshake.ClientHandshake;
import org.java_websocket.handshake.ServerHandshake;
import org.java_websocket.server.WebSocketServer;
import se.hellsoft.syncloud.SynCloudData;

import java.io.IOException;
import java.net.InetAddress;
import java.net.InetSocketAddress;
import java.net.URI;
import java.nio.ByteBuffer;
import java.util.Collections;
import java.util.HashSet;
import java.util.Set;

/**
 * @author Erik Hellman
 */
public class CommComponentTCP implements Handler.Callback {
    public static final String TAG = "CommComponentTCP";
    public static final int PORT = 9525;
    private static final int SEND_BROADCAST_MSG = 10;
    private static final int SEND_TIME_SYNC_REQUEST = 20;
    private static final int SEND_TIME_SYNC_RESPONSE = 30;

    private Handler mNetworkHandler;
    private Set<CommListener> mCommListeners;
    private MyServer mServer;
    private MyClient mClient;
    private Set<WebSocket> mClients;

    public CommComponentTCP() {
        mClients = Collections.synchronizedSet(new HashSet<WebSocket>());
        mCommListeners = Collections.synchronizedSet(new HashSet<CommListener>());
    }

    public void sendMessage(SynCloudData.SynMessage synMessage) {
        mNetworkHandler.obtainMessage(SEND_BROADCAST_MSG, synMessage).sendToTarget();
    }

    public void addListener(CommListener commListener) {
        mCommListeners.add(commListener);
    }

    public void removeListener(CommListener commListener) {
        mCommListeners.remove(commListener);
    }

    private void notifyListeners(SynCloudData.SynMessage synMessage) {
        for (CommListener commListener : mCommListeners) {
            commListener.onMessageReceived(synMessage);
        }
    }

    public void startServer(InetAddress wifiAddress) {
        HandlerThread handlerThread = new HandlerThread("NetworkSender");
        handlerThread.start();
        mNetworkHandler = new Handler(handlerThread.getLooper(), this);

        Log.d(TAG, "Start server on " + wifiAddress);
        mServer = new MyServer(new InetSocketAddress(wifiAddress, PORT));
        mServer.start();

    }

    public void stopServer() {
        try {
            mServer.stop();
        } catch (IOException e) {
            Log.e(TAG, "Error stopping server!", e);
        } catch (InterruptedException e) {
            Log.e(TAG, "Error stopping server!", e);
        }
    }

    public void connectClient(InetAddress hostAddress) {
        try {
            mClient = new MyClient(new URI("ws://" + hostAddress.getHostAddress() + ":" + PORT));
            mClient.connect();
        } catch (Exception e) {
            Log.e(TAG, "Error connecting to server!");
        }
    }

    public void disconnectClient() {
        mClient.close();
        mClient = null;
    }

    @Override
    public boolean handleMessage(Message msg) {
        byte[] msgData;
        SynCloudData.SynMessage synMessage;
        switch (msg.what) {
            case SEND_BROADCAST_MSG:
                synMessage = ((SynCloudData.SynMessage) msg.obj);
                msgData = synMessage.toByteArray();
                for (WebSocket client : mClients) {
                    client.send(msgData);
                }
                break;
            case SEND_TIME_SYNC_REQUEST:
                if(mClient != null) {
                    SynCloudData.SynMessage.Builder builder = SynCloudData.SynMessage.newBuilder();
                    builder.setType(SynCloudData.SynMessage.Type.TIME_SYNC_REQUEST);
                    long currentTime = SystemClock.elapsedRealtime();
                    builder.setTime(currentTime);
                    mClient.send(builder.build().toByteArray());
                }
                break;
            case SEND_TIME_SYNC_RESPONSE:
                break;
        }


        return true;
    }


    class MyClient extends WebSocketClient {

        public MyClient(URI serverURI) {
            super(serverURI);
        }

        @Override
        public void onOpen(ServerHandshake serverHandshake) {
            Log.d(TAG, "Connected to server!");
//            mNetworkHandler.sendEmptyMessageDelayed(SEND_TIME_SYNC_REQUEST, 2500);
        }

        @Override
        public void onMessage(String s) {
            // Ignored...
        }

        @Override
        public void onMessage(ByteBuffer bytes) {
            try {
                SynCloudData.SynMessage synMessage = SynCloudData.SynMessage.
                        parseFrom(ByteString.copyFrom(bytes));
                notifyListeners(synMessage);
            } catch (InvalidProtocolBufferException e) {
                Log.e(TAG, "Error decoding message!");
            }
        }

        @Override
        public void onClose(int i, String s, boolean b) {
            Log.d(TAG, "Disconnected from server!");
        }

        @Override
        public void onError(Exception e) {
            Log.e(TAG, "Error in communication!");
        }
    }

    class MyServer extends WebSocketServer {

        public MyServer(InetSocketAddress address) {
            super(address);
        }

        @Override
        public void onOpen(WebSocket webSocket, ClientHandshake clientHandshake) {
            mClients.add(webSocket);
        }

        @Override
        public void onClose(WebSocket webSocket, int i, String s, boolean b) {
            mClients.remove(webSocket);
        }

        @Override
        public void onMessage(WebSocket webSocket, String s) {
            Log.d(TAG, "Message received on server?");
        }

        @Override
        public void onError(WebSocket webSocket, Exception e) {
            Log.e(TAG, "Error in messaging...");
        }
    }

    public interface CommListener {
        void onMessageReceived(SynCloudData.SynMessage synMessage);
        void onClientConnected(WebSocket webSocket);
        void onTimeSync(long diffFromHost);
    }

}
