package se.hellsoft.syncloud.soundcloud;

import android.content.Context;
import android.media.MediaPlayer;
import android.net.Uri;
import android.os.Handler;
import android.os.HandlerThread;
import android.os.Message;
import android.os.SystemClock;
import android.util.Log;

import java.io.IOException;

/**
 * @author Erik Hellman
 */
public class SoundCloudPlayer implements Handler.Callback {
    private static final String TAG = "SoundCloudPlayer";
    public static final int PLAY_TRACK = 20;
    public static final int ENQUEUE_TRACK = 30;
    private final Handler mHandler;
    private Context mContext;
    private MediaPlayer mCurrentPlayer;
    private PlayerListener mPlayerListener;
    private MediaPlayer mCurrentMediaPlayer;

    public void setPlayerListener(PlayerListener playerListener) {
        mPlayerListener = playerListener;
    }

    public MediaPlayer getCurrentMediaPlayer() {
        return mCurrentMediaPlayer;
    }

    class EnqueueInfo {
        EnqueueInfo(Uri stream, long playtime) {
            this.stream = stream;
            this.playtime = playtime;
        }

        Uri stream;
        long playtime;
    }

    public SoundCloudPlayer(Context context) {
        mContext = context;

        HandlerThread handlerThread = new HandlerThread(TAG);
        handlerThread.start();
        mHandler = new Handler(handlerThread.getLooper(), this);
    }

    public void enqueueTrack(Uri stream, long playtime) {
        EnqueueInfo enqueueInfo = new EnqueueInfo(stream, playtime);
        mHandler.obtainMessage(ENQUEUE_TRACK, enqueueInfo).sendToTarget();
    }

    public void stopPlayback() {
        if(mCurrentPlayer != null) {
            mCurrentPlayer.release();
        }
        mCurrentPlayer = null;

        if(mPlayerListener != null) {
            mPlayerListener.onMediaPlayerChanged(null);
        }
    }


    @Override
    public boolean handleMessage(Message msg) {
        MediaPlayer mediaPlayer;
        switch (msg.what) {
            case PLAY_TRACK:
                if(mCurrentPlayer != null) {
                    mCurrentPlayer.release();
                }
                mCurrentPlayer = null;
                Log.d(TAG, "Play track!");
                mediaPlayer = (MediaPlayer) msg.obj;
                mediaPlayer.start();
                mCurrentPlayer = mediaPlayer;
                mPlayerListener.onMediaPlayerChanged(mCurrentPlayer);
                break;
            case ENQUEUE_TRACK:
                try {
                    EnqueueInfo enqueueInfo = (EnqueueInfo) msg.obj;
                    mediaPlayer = new MediaPlayer();
                    MediaListener mediaListener = new MediaListener(enqueueInfo.playtime);
                    mediaPlayer.setOnPreparedListener(mediaListener);
                    mediaPlayer.setOnBufferingUpdateListener(mediaListener);
                    mediaPlayer.setOnCompletionListener(mediaListener);
                    mediaPlayer.setOnErrorListener(mediaListener);
                    mediaPlayer.setDataSource(mContext.getApplicationContext(), enqueueInfo.stream);
                    mediaPlayer.prepareAsync();
                } catch (IOException e) {
                    Log.e(TAG, "Error preparing mediaplayer.", e);
                }
                break;
        }
        return true;
    }

    public interface PlayerListener {
        void onMediaPlayerChanged(MediaPlayer mediaPlayer);
    }

    class MediaListener implements MediaPlayer.OnCompletionListener,
            MediaPlayer.OnPreparedListener, MediaPlayer.OnBufferingUpdateListener, MediaPlayer.OnErrorListener {
        private long mPlaytime;
        private boolean mPrepared = false;
        private boolean mBuffered = false;
        private boolean mScheduled = false;
        MediaListener(long playtime) {
            mPlaytime = playtime;
        }

        @Override
        public void onPrepared(MediaPlayer mp) {
            mPrepared = true;
            if(mBuffered && !mScheduled) {
                Message message = mHandler.obtainMessage(PLAY_TRACK, mp);
                long now = SystemClock.elapsedRealtime();
                mHandler.sendMessageDelayed(message, mPlaytime - now);
                mScheduled = true;
            }
        }

        @Override
        public void onCompletion(MediaPlayer mp) {
            mp.reset();
            if(mPlayerListener != null) {
                mPlayerListener.onMediaPlayerChanged(mp);
            }
            mp.release();
        }

        @Override
        public void onBufferingUpdate(MediaPlayer mp, int percent) {
            mBuffered = percent > 70;

            if(mPrepared && mBuffered && !mScheduled) {
                Message message = mHandler.obtainMessage(PLAY_TRACK, mp);
                long now = SystemClock.elapsedRealtime();
                mHandler.sendMessageDelayed(message, mPlaytime - now);
                mScheduled = true;
            }
        }

        @Override
        public boolean onError(MediaPlayer mp, int what, int extra) {
            Log.d(TAG, "Error in mediaplayer: " + what + ", " + extra);
            return false;
        }
    }
}
